using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.IO.Ports;

namespace RoboFlash
{
    public partial class Form1 : Form

    {
        byte[] FlashBuffer = new byte[0x20000]; // create 128K flash buffer
        Int32 FileLength;
      
        SerialPort CommPort = new SerialPort();
        
        public Form1()
        {
            InitializeComponent();

        }

        private void Form1_Load(object sender, EventArgs e)
        {
            radioButton1.Checked = true;
            radioButton2.Checked = false;
            for (Int32 count = 0; count < 0x20000; count++) FlashBuffer[count] = 0xff;
            string[] ports = SerialPort.GetPortNames();

            foreach (string port in ports)
            {
                comboBox1.Items.Add(port);
            }
            comboBox1.SelectedIndex = 0;

        }

        private void button1_Click(object sender, EventArgs e)
        {
            // Create an OpenFileDialog object:
            OpenFileDialog openFile1 = new OpenFileDialog();

            // Initialize the OpenFileDialog to look for text files:

            if (radioButton1.Checked == true) openFile1.Filter = "AtMega Binary Files|*.bin";
            if (radioButton2.Checked == true) openFile1.Filter = "AtMega Hex Files|*.hex";
            // Check if the user selected a file from the OpenFileDialog:
            if (openFile1.ShowDialog() == System.Windows.Forms.DialogResult.OK)

                textBox1.Text = openFile1.FileName;

        }

        private void button2_Click(object sender, EventArgs e)
        {
            if (radioButton1.Checked == true) // binary file
            {
                FileInfo fi1 = new FileInfo(textBox1.Text);
                textBox3.Text += "Open Binary File" + Environment.NewLine;
                textBox3.Text += "File Size = " + fi1.Length + " Bytes" + Environment.NewLine;

                FileStream fs = new FileStream(textBox1.Text, FileMode.Open, FileAccess.Read);
                BinaryReader r = new BinaryReader(fs);
                for (long count = 0; count < fi1.Length; count++)
                {
                    FlashBuffer[count] = r.ReadByte();
                }
                textBox3.Text += "File Read Complete" + Environment.NewLine;
                r.Close();
                fs.Close();
                FileLength = (int)fi1.Length;
                if (fi1.Length >= 0xFD00)
                {
                    FileLength = 0;
                    MessageBox.Show("This File is too big for the RoboNova loader");
                }



            }

        }

        private void button3_Click(object sender, EventArgs e)
        {
            int Blocks, blockcount, wordcount;
            byte[] BytePoint = new byte[2];
            byte[] Checksum = new byte[1];
            byte[] DownLoad = new byte[256];
            CommPort.PortName = comboBox1.Text;
            CommPort.BaudRate = 9600;
            textBox3.Text += "Open Comm Port" + Environment.NewLine;
            try
            {
                CommPort.Open();
            }
            catch (Exception)
            {
                textBox3.Text += "Open Failed" + Environment.NewLine;
            }
            MessageBox.Show("Power Off RoboNova, click OK, then Power on RoboNova");
            CommPort.ReadTimeout = 10000;
            
            try
            {
               CommPort.ReadByte();
               if (CommPort.ReadByte() == 0x3E)
               {
                   byte[] Wakeup = { 0x3c, 0xF0, 0xA5, 0x5A, 0x0F, 0x80, 0x53 };
                   CommPort.Write(Wakeup, 0, 7);
                   byte[] Response = new byte[5];
                   CommPort.ReadByte();
                   CommPort.ReadByte();
                   CommPort.ReadByte();
                   CommPort.ReadByte();
                   CommPort.ReadByte();

                   if (((FileLength ) % 256) == 0) Blocks = (FileLength) / 256;
                   else Blocks = ((FileLength) / 256) + 1;
                   textBox3.Text += Blocks + " Blocks to write" + Environment.NewLine;
                   

                   for (blockcount = 0; blockcount < Blocks; blockcount++) 
                   {
                       BytePoint[0] = (byte)blockcount;
                       BytePoint[1] = 0;
                       Checksum[0] = (byte)BytePoint[0]; // Add high BytePoint to checksum
                  
                       for (wordcount = 0; wordcount < 128; wordcount++)
                       {
                           DownLoad[wordcount * 2] = FlashBuffer[(blockcount * 256) + (wordcount * 2)];
                           Checksum[0] += DownLoad[wordcount * 2];
                           DownLoad[wordcount * 2] ^= BytePoint[0];
                           DownLoad[(wordcount * 2) + 1] = FlashBuffer[(blockcount * 256) + (wordcount * 2) + 1];
                           Checksum[0] += DownLoad[(wordcount * 2) + 1];

                       }
                       
                       
                       CommPort.Write(BytePoint, 0, 2); // write BytePoint
                       CommPort.Write(DownLoad, 0, 256);
                       CommPort.Write(Checksum, 0, 1);
                       
                       CommPort.ReadByte(); // should check here if the correct 0x21 ack is received 
                     // or process the 0x40 error  by resend / abort
                   } 
               }
               else MessageBox.Show("Bad Response from Robo Nova");
            }
            catch
            {
                MessageBox.Show("NoResponse from RoboNova");
            }
            textBox3.Text += "Close Comm Port" + Environment.NewLine;
            CommPort.Close();
        }

    }

}